package ca.recrown.islandsurvivalcraft.interaction.items;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map.Entry;

import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.Event.Result;
import org.bukkit.event.block.Action;
import org.bukkit.event.entity.EntityPickupItemEvent;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.inventory.InventoryType.SlotType;
import org.bukkit.event.player.PlayerDropItemEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.player.PlayerItemHeldEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;

public class VariedItemManager implements Listener {
    private final IslandSurvivalCraft islandSurvivalCraft;

    public VariedItemManager(IslandSurvivalCraft islandSurvivalCraft) {
        this.islandSurvivalCraft = islandSurvivalCraft;
    }

    public void loadAllItems() {
        RegisteredVariedItem[] items = RegisteredVariedItem.values();
        for (int i = 0; i < items.length; i++) {
            if (items[i].getVariedItem().needInitialization()) {
                items[i].getVariedItem().initialize(islandSurvivalCraft);
            };
            islandSurvivalCraft.getServer().addRecipe(items[i].getVariedItem().getRecipe());
        }

        Collection<? extends Player> players = islandSurvivalCraft.getServer().getOnlinePlayers();
        for (Player player : players) {
            initializeInventoryVariedItems(player.getInventory());
        }
    }

    public void unloadAllItems() {
        RegisteredVariedItem[] items = RegisteredVariedItem.values();
        for (int i = 0; i < items.length; i++) {
            Bukkit.removeRecipe(items[i].getVariedItem().getNamespacedKey());
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onHeld(PlayerItemHeldEvent e) {
        ItemStack item = e.getPlayer().getInventory().getItem(e.getNewSlot());
        VariedItem variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            variedItem.onItemHeld(item, e.getPlayer());
        } else {
            item = e.getPlayer().getInventory().getItem(e.getPreviousSlot());
            variedItem = getVariedItemFromItemStack(item);
            if (variedItem != null) {
                variedItem.onItemPutAway(item, e.getPlayer());
            }
        }
    }

    @EventHandler(priority = EventPriority.LOWEST)
    public void onInventoryInteract(InventoryClickEvent e) {
        ItemStack item = e.getCurrentItem();
        VariedItem variedItem = getVariedItemFromItemStack(item);
        if (e.getSlotType() == SlotType.RESULT) {
            if (variedItem != null) {
                if (variedItem.onItemCrafted(item, e.getWhoClicked())) {
                    e.setCancelled(true);
                }            
            }
        } else if (e.getSlotType() == SlotType.QUICKBAR) {
            if (e.getWhoClicked().getInventory().getHeldItemSlot() == e.getSlot()) {
                if (variedItem != null) {
                    variedItem.onItemPutAway(item, e.getWhoClicked());
                } else if ((variedItem = getVariedItemFromItemStack(e.getCursor())) != null) {
                    variedItem.onItemHeld(item, e.getWhoClicked());
                }
            }
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onPlayerDropItem(PlayerDropItemEvent e) {
        ItemStack item = e.getItemDrop().getItemStack();
        VariedItem variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            variedItem.onItemPutAway(item, e.getPlayer());
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onEntityPickup(EntityPickupItemEvent e) {
        ItemStack item = e.getItem().getItemStack();
        System.out.println("PICKUP");
        VariedItem variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            if (e.getEntityType() == EntityType.PLAYER) {
                Player player = (Player) e.getEntity();
                if (player.getInventory().getHeldItemSlot() == player.getInventory().firstEmpty()) {
                    variedItem.onItemHeld(item, player);
                }
            }
        }
    }

    @EventHandler(priority = EventPriority.LOWEST)
    public void onPlayerInteraction(PlayerInteractEvent e) {
        if (e.useItemInHand() == Result.DENY || (e.getAction() != Action.RIGHT_CLICK_AIR && e.getAction() != Action.RIGHT_CLICK_BLOCK)) return;
        ItemStack item = e.getItem();
        VariedItem variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            if(variedItem.onItemRightClicked(item, e.getPlayer(), e.getHand(), e.getClickedBlock(), e.getBlockFace())) {
                e.setCancelled(true);
                e.setUseItemInHand(Result.DENY);
            }
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onPlayerJoin(PlayerJoinEvent e) {
        initializeInventoryVariedItems(e.getPlayer().getInventory());
        ItemStack item = e.getPlayer().getInventory().getItemInMainHand();
        VariedItem variedItem = getVariedItemFromItemStack(item);
        if (variedItem != null) {
            variedItem.onItemHeld(item, e.getPlayer());
        }
    }

    private void initializeInventoryVariedItems(Inventory inv) {
        RegisteredVariedItem[] registeredVariedItems = RegisteredVariedItem.values();
        for (RegisteredVariedItem registeredVariedItem : registeredVariedItems) {
            VariedItem variedItem = registeredVariedItem.getVariedItem();
            
            Material[] bases = variedItem.getBaseItems();
            for (Material base : bases) {
                HashMap<Integer, ? extends ItemStack> potentials = inv.all(base);
                for (Entry<Integer, ? extends ItemStack> entry : potentials.entrySet()) {
                    String identifier = entry.getValue().getItemMeta().getPersistentDataContainer().get(variedItem.getNamespacedKey(), variedItem.getIdentifier());
                    if (identifier != null) {
                        variedItem.initializeInstanceOfItem(entry.getValue());
                    }
                }
            }

        }
    }

    private VariedItem getVariedItemFromItemStack(ItemStack item) {
        if (item == null || item.getType() == Material.AIR) return null;
        ItemMeta meta = item.getItemMeta();
        RegisteredVariedItem[] items = RegisteredVariedItem.values();
        for (int i = 0; i < items.length; i++) {
            VariedItem curr = items[i].getVariedItem();
            String identifier = meta.getPersistentDataContainer().get(curr.getNamespacedKey(), curr.getIdentifier());
            if (identifier != null) {
                if (curr.getIdentifier().getID().equals(identifier)) {
                    return curr;
                }
            }
        }

        return null;
    }
}