package ca.recrown.islandsurvivalcraft.interaction.items.variations;

import java.awt.Color;
import java.util.ArrayList;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.GameMode;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.entity.HumanEntity;
import org.bukkit.entity.Player;
import org.bukkit.inventory.EquipmentSlot;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.Recipe;
import org.bukkit.inventory.ShapelessRecipe;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.MapMeta;
import org.bukkit.map.MapCanvas;
import org.bukkit.map.MapPalette;
import org.bukkit.map.MapRenderer;
import org.bukkit.map.MapView;
import org.bukkit.map.MapView.Scale;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;
import ca.recrown.islandsurvivalcraft.interaction.items.VariationItemIdentifier;
import ca.recrown.islandsurvivalcraft.interaction.items.VariedItem;
import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;
import ca.recrown.islandsurvivalcraft.world.WorldInfo;
import ca.recrown.islandsurvivalcraft.world.Information.IslandInformation;
import ca.recrown.islandsurvivalcraft.world.Information.IslandInformationManager;

public class IslandMapItem extends MapRenderer implements VariedItem {
    private Material[] baseItems;
    private VariationItemIdentifier identifier;
    private IslandSurvivalCraft islandSurvivalCraft;
    private NamespacedKey namespacedKey;
    private ShapelessRecipe recipe;
    private ArrayList<String> lore = new ArrayList<>();

    @Override
    public void initialize(IslandSurvivalCraft islandSurvivalCraft) {
        lore.add(ChatColor.RESET.toString() + ChatColor.GRAY + "Land ho!");
        this.islandSurvivalCraft = islandSurvivalCraft;
        baseItems = new Material[2];
        baseItems[0] = Material.MAP;
        baseItems[1] = Material.FILLED_MAP;
        namespacedKey = new NamespacedKey(islandSurvivalCraft, "island_map");
        ItemStack resultItem = new ItemStack(getBaseItems()[0], 1);
        ItemMeta metadata = resultItem.getItemMeta();
        metadata.getPersistentDataContainer().set(getNamespacedKey(), getIdentifier(), getIdentifier().getID());
        metadata.setLore(lore);
        resultItem.setItemMeta(metadata);
        recipe = new ShapelessRecipe(namespacedKey, resultItem);
        recipe.addIngredient(Material.REDSTONE);
        recipe.addIngredient(new ItemStack(Material.MAP));
    }

    @SuppressWarnings("deprecation")
    @Override
    public void render(MapView map, MapCanvas canvas, Player player) {
        WorldInfo worldInfo = islandSurvivalCraft.getWorldInfoManager().retrieve(map.getWorld().getName());
        IslandInformationManager islandInformationManager = worldInfo.getIslandInfoManager();
        int blocksPerPixel = (int) (1 * Math.pow(2, map.getScale().getValue()));
        for (int x = 0; x < 128; x ++) {
            for (int y = 0; y < 128; y ++) {
                int actualX = blocksPerPixel * (x - Byte.MAX_VALUE/2) + map.getCenterX();
                int actualZ = blocksPerPixel * (y - Byte.MAX_VALUE/2) + map.getCenterZ();
                byte baseColorVal = canvas.getBasePixel(x, y);
                Color baseColor = MapPalette.getColor(baseColorVal);
                Point2 pCoords = new Point2(actualX, actualZ);
                IslandInformation info = islandInformationManager.getIslandInformation(pCoords, false);
                if (info != null) {
                    Color mixed = new Color((int) Math.min(255, baseColor.getRed() * 1.5f), (int) Math.min(255, baseColor.getGreen() * 0.7f), (int) Math.min(255, baseColor.getBlue() * 0.44f));
                    canvas.setPixel(x, y, MapPalette.matchColor(mixed));
                }
            }
        }
    }

    @Override
    public Recipe getRecipe() {
        return recipe;
    }

    @Override
    public String getName() {
        return "Island Map";
    }

    @Override
    public Material[] getBaseItems() {
        return baseItems;
    }

    @Override
    public NamespacedKey getNamespacedKey() {
        return namespacedKey;
    }

    @Override
    public boolean onItemCrafted(ItemStack item, HumanEntity crafter) {
        if (item.getType() != Material.FILLED_MAP) return false;
        Bukkit.getScheduler().scheduleSyncDelayedTask(islandSurvivalCraft, () -> {
            initializeInstanceOfItem(item);
        });
        return false;
    }

    @Override
    public void assignIdentifier(VariationItemIdentifier identifier) {
        this.identifier = identifier;
    }

    @Override
    public VariationItemIdentifier getIdentifier() {
        return identifier;
    }

    @Override
    public boolean needInitialization() {
        return namespacedKey == null;
    }

    @Override
    public boolean onItemRightClicked(ItemStack item, HumanEntity clicker, EquipmentSlot hand, Block block,
            BlockFace blockFace) {
                if (item.getType() != Material.MAP) return false;
                ItemStack freshMap = new ItemStack(Material.FILLED_MAP, 1);
                MapMeta mapMeta = (MapMeta) freshMap.getItemMeta();
                mapMeta.setLore(lore);
                mapMeta.getPersistentDataContainer().set(getNamespacedKey(), getIdentifier(), getIdentifier().getID());
                MapView mapView = Bukkit.createMap(clicker.getWorld());
                mapView.setTrackingPosition(true);
                mapView.setCenterX(Math.round(clicker.getLocation().getBlockX() / 128f) * 128);
                mapView.setCenterZ(Math.round(clicker.getLocation().getBlockZ() / 128f) * 128);
                mapView.setScale(Scale.CLOSEST);
                mapMeta.setMapView(mapView);
                freshMap.setItemMeta(mapMeta);
                initializeInstanceOfItem(freshMap);
                if (clicker.getGameMode() != GameMode.CREATIVE) {
                    clicker.getInventory().setItem(hand, freshMap);
                } else {
                    clicker.getInventory().addItem(freshMap);
                }
        return true;
    }

    @Override
    public void initializeInstanceOfItem(ItemStack item) {
        if (item.getType() != Material.FILLED_MAP) return;
        MapMeta mapMeta = (MapMeta) item.getItemMeta();
        MapView mapView = mapMeta.getMapView();
        mapView.removeRenderer(this);
        mapView.addRenderer(this);
        mapMeta.setMapView(mapView);
        item.setItemMeta(mapMeta);
    }

    @Override
    public void onItemHeld(ItemStack heldItem, HumanEntity holder) {
    }

    @Override
    public void onItemPutAway(ItemStack item, HumanEntity holder) {
    }

    @Override
    public String getDescription() {
        return "A map that has island hunting properties. Will highlight islands in relation on map.";
    }
}