package ca.recrown.islandsurvivalcraft.utilities.datatypes;

import java.util.Objects;

public class Point2 {
    public final int x, y;
    private final int hash;

    public Point2(int x, int y) {
        this.x = x;
        this.y = y;
        this.hash = Objects.hash(x, y);
    }

    public Point2(Point2 point) {
        this(point.x, point.y);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Point2) {
            Point2 p = (Point2) obj;
            return p.x == this.x && p.y == this.y;
        }
        return false;
    }

    public boolean fastEquals(Point2 point) {
        return point.hashCode() == this.hash && x == point.x && y == point.y;
    }

    public double distance(Point2 point) {
        return Math.hypot(point.x - this.x, point.y - this.y);
    }

    @Override
    public int hashCode() {
        return hash;
    }

    @Override
    public String toString() {
        return String.format("(%d, %d)", x, y);
    }

    public Point2 shift(int x, int y) {
        return new Point2(this.x + x, this.y + y);
    }
}