package ca.recrown.islandsurvivalcraft.utilities.drawing;

import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;
import ca.recrown.islandsurvivalcraft.utilities.pathfinding.CoordinateValidatable;

public class Circle {
    public final Point2 center;
    public final CoordinateValidatable drawer;
    public final int r;
    int p, x, y;

    /**
     * A circle drawing object.
     * @param center The center point.
     * @param radius The radius of the circle to be drawn.
     * @param drawer The coordinate validator to be used to draw. The value the validator returns is ignored.
     */
    public Circle(Point2 center, int radius, CoordinateValidatable drawer) {
        if (center == null) throw new NullPointerException("Center cannot be null.");
        if (radius <= 0) throw new IllegalArgumentException("radius cannot be 0 or less.");
        if (drawer == null) throw new NullPointerException("Drawer cannot be null.");
        this.center = center;
        this.r = radius;
        this.drawer = drawer;

        p = 1 - radius;
        x = 0;
        y = this.r;
    }

    /**
     * 
     * @return true if done.
     */
    public boolean step() {
        if (x > y) return true;
        drawer.validate(new Point2(center.x + x, center.y + y));
        drawer.validate(new Point2(center.x + x, center.y - y));
        drawer.validate(new Point2(center.x - x, center.y + y));
        drawer.validate(new Point2(center.x - x, center.y - y));
        drawer.validate(new Point2(center.x + y, center.y + x));
        drawer.validate(new Point2(center.x + y, center.y - x));
        drawer.validate(new Point2(center.x - y, center.y + x));
        drawer.validate(new Point2(center.x - y, center.y - x));
        if (p < 0) {
            p += 2*x + 3;
        } else {
            p += 2 * x - 2 * y + 5;
            y--;
        }
        x++;
        return x > y;
    }

    public void draw() {
        while (!step());
    }
}