package ca.recrown.islandsurvivalcraft.utilities.drawing;


import java.util.HashSet;
import java.util.LinkedList;

import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;
import ca.recrown.islandsurvivalcraft.utilities.pathfinding.CoordinateValidatable;

public class Flooder {
    private final Point2 start;
    private final CoordinateValidatable floodable;

    public Flooder(Point2 start, CoordinateValidatable floodable) {
        this.start = start;
        this.floodable = floodable;
    }

    public Flooder(int xStart, int yStart, CoordinateValidatable floodable) {
        this(new Point2(xStart, yStart),  floodable);
    }

    public void start() {
        HashSet<Point2> checked = new HashSet<>();
        LinkedList<Point2> queue = new LinkedList<>();

        if (!checked.add(start)) return;
        if (!floodable.validate(start)) return;
        queue.add(start);
        while (!queue.isEmpty()) {
            Point2 p = queue.pop();

            Point2 pE = new Point2(p.x + 1, p.y);
            if (checked.add(pE) && floodable.validate(pE)) queue.add(pE);

            Point2 pW = new Point2(p.x - 1, p.y);
            if (checked.add(pW) && floodable.validate(pW)) queue.add(pW);

            Point2 pN = new Point2(p.x, p.y + 1);
            if (checked.add(pN) && floodable.validate(pN)) queue.add(pN);

            Point2 pS = new Point2(p.x, p.y - 1);
            if (checked.add(pS) && floodable.validate(pS)) queue.add(pS);
        }
    }
}