package ca.recrown.islandsurvivalcraft.world.Information;

import java.util.Collections;
import java.util.Set;
import java.util.UUID;

import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;

public class IslandInformation {
    private final Set<Point2> chunkSpan;
    private final Set<Point2> islandCoordinates;
    private final Set<Point2> edgeCoordinates;
    private final Point2 islandOrigin;
    private UUID ownerUUID;
    
    public IslandInformation(Point2 islandOrigin, Set<Point2> islandCoordinates, Set<Point2> edgeCoordinates, Set<Point2> chunksUsed) {
        this.islandCoordinates = Collections.unmodifiableSet(islandCoordinates);
        this.chunkSpan = Collections.unmodifiableSet(chunksUsed);
        this.edgeCoordinates = Collections.unmodifiableSet(edgeCoordinates);
        this.islandOrigin = islandOrigin;
    }

    /**
     * @return a set that represents all coordinates within this island.
     */
    public Set<Point2> getIslandCoordinates() {
        return islandCoordinates;
    }

    /**
     * @return a set of chunks that this island spans.
     */
    public Set<Point2> getChunkSpan() {
        return chunkSpan;
    }

    /**
     * @return a set of points that represent all the edges of this island.
     */
    public Set<Point2> getEdgeCoordinates() {
        return edgeCoordinates;
    }

    /**
     * @param ownerUUID the UUID of the player owner of this island. May be null.
     */
    public void setOwnerUUID(UUID ownerUUID) {
        this.ownerUUID = ownerUUID;
    }

    /**
     * @return the islandOrigin
     */
    public Point2 getIslandOrigin() {
        return islandOrigin;
    }

    /**
     * @return the UUID of the player owner of this island. May be null.
     */
    public UUID getOwnerUUID() {
        return ownerUUID;
    }

    public boolean isWithinIsland(Point2 coordinates) {
        return islandCoordinates.contains(coordinates);
    }

    public boolean isEdgeOfIsland(Point2 coordinates) {
        return edgeCoordinates.contains(coordinates);
    }

    public boolean isIslandInChunk(Point2 chunkCoords) {
        return chunkSpan.contains(chunkCoords);
    }

    @Override
    public int hashCode() {
        return islandOrigin.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof IslandInformation) {
            return islandOrigin.equals(((IslandInformation) obj).getIslandOrigin());
        }
        return false;
    }
}