package ca.recrown.islandsurvivalcraft.world;

import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang.NullArgumentException;
import org.bukkit.Chunk;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.world.ChunkLoadEvent;
import org.bukkit.event.world.ChunkUnloadEvent;
import org.bukkit.generator.ChunkGenerator;

import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;
import ca.recrown.islandsurvivalcraft.world.generation.GeneratorModes;
import ca.recrown.islandsurvivalcraft.world.generation.chunks.IslandWorldChunkGenerator;

public class WorldInfoManager implements Listener {
    public final ConcurrentHashMap<String, WorldInfo> worldInformation = new ConcurrentHashMap<>();

    public WorldInfoManager() {
    }
    /**
     * Return the world info requested for the given world.
     * Should not be null.
     * @param world The world the associated info is to be requested for.
     * @return The world info.
     */
    public WorldInfo retrieve(String worldName) {
        if (worldName == null) throw new NullArgumentException("world");
        return worldInformation.computeIfAbsent(worldName, (name) -> {
            return new WorldInfo(this);
        });
    }

    public ChunkGenerator getChunkGenerator(String worldName, GeneratorModes mode) {
        IslandWorldChunkGenerator islandWorldChunkGenerator = retrieve(worldName).getGenerator();
        if (!islandWorldChunkGenerator.isInitialized()) islandWorldChunkGenerator.setGeneratorType(mode);
        return islandWorldChunkGenerator;
    }

    /**
     * Resets the manager deleting all currently loaded world info from memory.
     */
    public void reset() {
        worldInformation.clear();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onChunkLoad(ChunkLoadEvent event) {
        Chunk chunk = event.getChunk();
        if (!worldInformation.containsKey(event.getWorld().getName())) return;
        WorldInfo worldInfo = retrieve(event.getWorld().getName());
        if (!worldInfo.isInitialized()) worldInfo.initialize(event.getWorld());
        worldInfo.getIslandInfoManager().loadChunkIslandInformationAsync(new Point2(chunk.getX(), chunk.getZ()));
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onChunkUnload(ChunkUnloadEvent event) {
        Chunk chunk = event.getChunk();
        if (!worldInformation.containsKey(event.getWorld().getName())) return;
        WorldInfo worldInfo = retrieve(event.getWorld().getName());
        if (!worldInfo.isInitialized()) worldInfo.initialize(event.getWorld());
        worldInfo.getIslandInfoManager().unloadChunkIslandInformation(new Point2(chunk.getX(), chunk.getZ()));
    }
}